/**********************************************************************
 * Copyright (c) 2005 Scapa Technologies Limited and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * Scapa Technologies Limited - Initial API and implementation
 **********************************************************************/

package org.eclipse.stp.b2j.core.jengine.internal.core.bpel;

import java.util.ArrayList;

import org.eclipse.stp.b2j.core.jengine.internal.compiler.xmlns.NamespaceTranslator;
import org.eclipse.stp.b2j.core.jengine.internal.utils.CharStack;

/**
 * 
 * @author amiguel
 * 
 * Part of the generic runtime engine API, used to help implement BPEL messaging activities.
 */
public class WSEndpointReference {

	String address;
	ArrayList ref_properties = new ArrayList();
	String port_type;
	String service_name;
	String service_port;
	ArrayList policies = new ArrayList();
	
	private WSEndpointReference() {
	}
	
	public WSEndpointReference(String address) {
		this.address = address;
	}
	
	public String getAddress() {
		return address;
	}
	
	//
	// Use CharStack in here to convert to and from
	//
	public static WSEndpointReference fromXML(String xml) throws Exception {
		WSEndpointReference epr = new WSEndpointReference();
		CharStack cs = new CharStack(xml);
		String elem_epr = cs.popXmlElementStart();
		if (!elem_epr.endsWith("EndpointReference")) {
			throw new Exception("expected tag 'EndpointReference', instead found '"+elem_epr+"'");
		}
		cs.popXmlElementEnd();
		while (true) {
			String elem = cs.popXmlElementStart();
			if (elem.endsWith("Address")) {
				cs.popXmlAttributes();
				cs.popXmlElementEnd();
				
				epr.address = cs.popXmlText(true,true);
				
			} else if (elem.endsWith("ServiceName")) {
				
				String attr = cs.popXmlAttributeName();
				while (attr.length() > 0) {
					if (attr.equals("PortName")) {
						epr.service_port = cs.popXmlAttributeValue();
					} else {
						cs.popXmlAttributeValue();
					}
					attr = cs.popXmlAttributeName();
				}

				cs.popXmlElementEnd();
				
				epr.service_name = cs.popXmlText(true);
				
			} else if (elem.endsWith("PortType")) {
				cs.popXmlAttributes();
				cs.popXmlElementEnd();
				
				epr.port_type = cs.popXmlText(true);
				
			} else if (elem.endsWith("ReferenceProperties")) {
				cs.popXmlAttributes();
				cs.popXmlElementEnd();
				
				epr.ref_properties.add(cs.popXmlSubTree(true));
				
			} else if (elem.endsWith("EndpointReference") && elem.startsWith("/")) {
				cs.popXmlElementEnd();
				break;
			} else {
				//unrecognised element
				cs.popXmlAttributes();
				cs.popXmlElementEnd();
			}
			//pop end of this element
			cs.popXmlElementStart();
			cs.popXmlElementEnd();
		}
		return epr;
	}
	public static String toXML(WSEndpointReference epr) {
		StringBuffer sb = new StringBuffer();
		sb.append("<wsa:EndpointReference xlmns:wsa=\"").append(NamespaceTranslator.NAMESPACE_WSA).append("\">");
		sb.append("<wsa:Address>").append(epr.address).append("</wsa:Address>");
		if (epr.port_type != null) {
			sb.append("<wsa:PortType>").append(epr.port_type).append("</wsa:PortType>");
		}
		if (epr.service_name != null) {
			if (epr.service_port != null) {
				sb.append("<wsa:ServiceName>").append(epr.service_name).append("</wsa:ServiceName>");
			} else {
				sb.append("<wsa:ServiceName ");
				sb.append("PortName=\"");
				sb.append(epr.service_port);
				sb.append("\" >");
				sb.append(epr.service_name);
				sb.append("</wsa:ServiceName>");
			}
		}
		for (int i = 0; i < epr.ref_properties.size(); i++) {
			sb.append("<wsa:ReferenceProperties>");
			sb.append((String)epr.ref_properties.get(i));
			sb.append("</wsa:ReferenceProperties>");
		}
		sb.append("</wsa:EndpointReference>");
		return sb.toString();
	}
	
	public String toString() {
		return toXML(this);
	}
	
	public static void main(String[] args) {
		try {
			System.out.println(fromXML("<EndpointReference xmlns=\"http://schemas.xmlsoap.org/ws/2004/03/addressing\" ><!--LINENO file (Scapa Merged Choreography BPEL) line 43 --><Address ><!--LINENO file (Scapa Merged Choreography BPEL) line 43 -->http://localhost:30291/</Address></EndpointReference>"));			
			
			StringBuffer sb = new StringBuffer();
			sb.append("<EndpointReference>");
			sb.append("<Address>blah</Address>");
			sb.append("</EndpointReference>");
			System.out.println(fromXML(sb.toString()));
			sb = new StringBuffer();
			sb.append("<EndpointReference>");
			sb.append("<Address>myAddress</Address>");
			sb.append("<PortType>myPortType</PortType>");
			sb.append("<ServiceName PortName=\"myPortName\">myServiceName</ServiceName>");
			sb.append("<ReferenceProperties>aaaa<bbbb>cccc</bbbb>dddd</ReferenceProperties>");
			sb.append("</EndpointReference>");
			System.out.println(fromXML(sb.toString()));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}